<template>
  <ContentWrap>
    <!-- 搜索工作栏 -->
    <el-form
      class="-mb-15px"
      :model="queryParams"
      ref="queryFormRef"
      :inline="true"
      label-width="68px"
    >
      <el-form-item label="主机名称" prop="hostName">
        <el-select v-model="queryParams.hostName" placeholder="请选择" class="!w-180px" @change="getDataList">
          <el-option
            v-for="(host, index) in hosts"
            :key="index"
            :label="host"
            :value="host"
          />
        </el-select>
      </el-form-item>
      <el-form-item label="服务器IP" prop="hostIp">
        <el-select v-model="queryParams.hostIp" clearable placeholder="请选择" class="!w-160px" @change="getDataList">
          <el-option
            v-for="(ip, index) in ips"
            :key="index"
            :label="ip"
            :value="ip"
          />
        </el-select>
      </el-form-item>
      <el-form-item label="服务名" prop="serverName">
        <el-select v-model="queryParams.serverName" placeholder="请选择" class="!w-160px" @change="getDataList">
          <el-option
            v-for="(server, index) in servers"
            :key="index"
            :label="server"
            :value="server"
          />
        </el-select>
      </el-form-item>
      <el-form-item label="创建时间" prop="createTime">
        <el-date-picker
          v-model="queryParams.createTime"
          value-format="YYYY-MM-DD HH:mm:ss"
          type="datetimerange"
          start-placeholder="开始日期"
          end-placeholder="结束日期"
          :default-time="[new Date('1 00:00:00'), new Date('1 23:59:59')]"
          class="!w-360px"
        />
      </el-form-item>
      <el-form-item>
        <el-button @click="handleQuery">
          <Icon icon="ep:search" class="mr-5px"/>
          搜索
        </el-button>
        <el-button @click="resetQuery">
          <Icon icon="ep:refresh" class="mr-5px"/>
          重置
        </el-button>
        <el-button
          type="primary"
          plain
          @click="openForm('create')"
          v-hasPermi="['infra:monitor-mem:create']"
        >
          <Icon icon="ep:plus" class="mr-5px"/>
          新增
        </el-button>
        <el-button
          type="success"
          plain
          @click="handleExport"
          :loading="exportLoading"
          v-hasPermi="['infra:monitor-mem:export']"
        >
          <Icon icon="ep:download" class="mr-5px"/>
          导出
        </el-button>
      </el-form-item>
      <el-form-item style="float: right">
        <el-button
          v-if="showType == 'chart'"
          type="warning"
          style="font-weight: bold"
          plain
          @click="switchShow('data')">
          <Icon icon="fa-solid:th-list" class="mr-5px"/>
          列表展示
        </el-button>
        <el-button
          v-if="showType == 'data'"
          type="danger"
          style="font-weight: bold"
          plain
          @click="switchShow('chart')">
          <Icon icon="fa-solid:chart-pie" class="mr-5px"/>
          图例展示
        </el-button>
      </el-form-item>
    </el-form>
  </ContentWrap>

  <ContentWrap v-if="showType == 'chart'">
    <!-- 物理内存折线图 -->
    <el-skeleton :loading="echartsLoading" animated>
      <Echart :height="320" :options="physicalChartOptions"/>
    </el-skeleton>
    <!-- JVM内存折线图 -->
    <el-skeleton :loading="echartsLoading" animated>
      <Echart style="margin-top: 20px" :height="320" :options="JVMChartOptions"/>
    </el-skeleton>
  </ContentWrap>

  <!-- 列表 -->
  <ContentWrap v-if="showType == 'data'">
    <el-table v-loading="loading" :data="list" :stripe="true" :show-overflow-tooltip="true">
      <el-table-column label="主机名称" align="center" prop="hostName" width="160"/>
      <el-table-column label="服务器ip" align="center" prop="hostIp" width="150"/>
      <el-table-column label="服务名" align="center" prop="serverName" width="120"/>
      <el-table-column label="总内存" align="center" prop="physicalTotal"/>
      <el-table-column label="已用内存" align="center" prop="physicalUsed"/>
      <el-table-column label="空闲内存" align="center" prop="physicalFree"/>
      <el-table-column label="内存使用率" align="center" prop="physicalUsage"/>
      <el-table-column label="JVM占用内存" align="center" prop="runtimeTotal"/>
      <el-table-column label="JVM最大内存" align="center" prop="runtimeMax"/>
      <el-table-column label="JVM可用内存" align="center" prop="runtimeUsed"/>
      <el-table-column label="JVM空闲内存" align="center" prop="runtimeFree"/>
      <el-table-column label="JVM内存使用率" align="center" prop="runtimeUsage"/>
      <el-table-column
        label="创建时间"
        align="center"
        prop="createTime"
        :formatter="dateFormatter"
        width="180px"
      />
      <el-table-column label="操作" align="center" width="120">
        <template #default="scope">
          <el-button
            link
            type="primary"
            @click="openForm('update', scope.row.id)"
            v-hasPermi="['infra:monitor-mem:query']"
          >
            详情
          </el-button>
          <el-button
            link
            type="danger"
            @click="handleDelete(scope.row.id)"
            v-hasPermi="['infra:monitor-mem:delete']"
          >
            删除
          </el-button>
        </template>
      </el-table-column>
    </el-table>
    <!-- 分页 -->
    <Pagination
      :total="total"
      v-model:page="queryParams.pageNo"
      v-model:limit="queryParams.pageSize"
      @pagination="getList"
    />
  </ContentWrap>

  <!-- 表单弹窗：添加/修改 -->
  <MonitorMemForm ref="formRef" @success="getList"/>
</template>

<script setup lang="ts">
import {dateFormatter} from '@/utils/formatTime'
import download from '@/utils/download'
import {MonitorMemApi, MonitorMemVO} from '@/api/infra/monitormem'
import MonitorMemForm from './MonitorMemForm.vue'
import {EChartsOption} from "echarts";
import {formatTime} from "@/utils";
import {formatDate} from "@vueuse/core";

/** 内存监控日志 列表 */
defineOptions({name: 'MonitorMem'})

const message = useMessage() // 消息弹窗
const {t} = useI18n() // 国际化

const loading = ref(true) // 列表的加载中
const list = ref<MonitorMemVO[]>([]) // 列表的数据
const hosts = ref<String[]>([]) // 主机列表
const ips = ref<String[]>([]) // ip列表
const servers = ref<String[]>([]) // 服务列表
const total = ref(0) // 列表的总页数
const queryParams = reactive({
  pageNo: 1,
  pageSize: 10,
  hostName: undefined,
  hostIp: undefined,
  serverName: undefined,
  physicalTotal: undefined,
  physicalUsed: undefined,
  physicalFree: undefined,
  physicalUsage: undefined,
  runtimeTotal: undefined,
  runtimeMax: undefined,
  runtimeUsed: undefined,
  runtimeFree: undefined,
  runtimeUsage: undefined,
  createTime: [],
})
const queryFormRef = ref() // 搜索的表单
const exportLoading = ref(false) // 导出的加载中
const echartsLoading = ref(true) // 图表加载中
const showType = ref() //展示类型（chart-图例，data-数据）

/** 查询列表 */
const getList = async () => {
  loading.value = true
  try {
    const data = await MonitorMemApi.getMonitorMemPage(queryParams)
    list.value = data.list
    total.value = data.total
  } finally {
    loading.value = false
  }
}

/** 查询所有主机名 */
const getAllHost = async () => {
  const data = await MonitorMemApi.getAllHost()
  // 确保 data 存在且非空后，设置默认选中第一个
  if (data.length > 0) {
    queryParams.hostName = data[0];
  }
  hosts.value = data
}

/** 查询所有ip */
const getAllIp = async () => {
  const data = await MonitorMemApi.getAllIp()
  ips.value = data
}

/** 查询所有服务名 */
const getAllServer = async () => {
  const data = await MonitorMemApi.getAllServer()
  servers.value = data
}

/** 封装接口调用 */
const getDataList = async () => {
  if(showType.value == 'data') {
    await getList()
  } else {
    dealDate()
    await getMonitorMemDataList()
  }
}

/** 处理查询时间段 */
const dealDate = async () => {
  const currentDate = new Date();
  const previousDate = new Date(currentDate);
  previousDate.setDate(currentDate.getDate() - 1);
  queryParams.createTime = []
  queryParams.createTime[0] = formatDate(previousDate, 'YYYY-MM-DD HH:mm:ss');
  queryParams.createTime[1] = formatDate(currentDate, 'YYYY-MM-DD HH:mm:ss');
}

/** 搜索按钮操作 */
const handleQuery = () => {
  queryParams.pageNo = 1
  getDataList()
}

/** 重置按钮操作 */
const resetQuery = () => {
  queryFormRef.value.resetFields()
  if (hosts.value.length > 0) {
    queryParams.hostName = hosts.value[0];
  }
  if(showType.value != 'data') {
    if (servers.value.length > 0) {
      queryParams.serverName = servers.value[0];
    }
  }
  handleQuery()
}

/** 添加/修改操作 */
const formRef = ref()
const openForm = (type: string, id?: number) => {
  formRef.value.open(type, id)
}

/** 删除按钮操作 */
const handleDelete = async (id: number) => {
  try {
    // 删除的二次确认
    await message.delConfirm()
    // 发起删除
    await MonitorMemApi.deleteMonitorMem(id)
    message.success(t('common.delSuccess'))
    // 刷新列表
    await getList()
  } catch {
  }
}

/** 堆叠面积图配置 */
const physicalChartOptions = reactive<EChartsOption>({
  title: {
    text: '物理内存折线图'
  },
  dataset: {
    dimensions: ['createTime', 'physicalTotal', 'physicalUsed', 'physicalFree'],
    source: []
  },
  grid: {
    left: 20,
    right: 20,
    bottom: 10,
    top: 70,
    containLabel: true
  },
  legend: {
    top: 0
  },
  series: [
    {
      name: '总物理内存', type: 'line',
      emphasis: {
        focus: 'series'
      }, smooth: false
    },
    {
      name: '已用物理内存', type: 'line', areaStyle: {},
      emphasis: {
        focus: 'series'
      }, smooth: false
    },
    {
      name: '剩余物理内存', type: 'line', areaStyle: {},
      emphasis: {
        focus: 'series'
      }, smooth: false
    }
  ],
  toolbox: {
    feature: {
      // 数据区域缩放
      dataZoom: {
        yAxisIndex: false // Y轴不缩放
      },
      brush: {
        type: ['lineX', 'clear'] // 区域缩放按钮、还原按钮
      },
      saveAsImage: {show: true, name: '物理内存日志图片'} // 保存为图片
    }
  },
  tooltip: {
    trigger: 'axis',
    axisPointer: {
      type: 'cross',
      label: {
        backgroundColor: '#6a7985'
      }
    },
    padding: [5, 10]
  },
  xAxis: {
    type: 'category',
    boundaryGap: false,
    axisTick: {
      show: false
    }
  },
  yAxis: {
    name: "单位（MB）",
    nameTextStyle: {
      color: "#aaa",
      nameLocation: "start",
    },
  },
}) as EChartsOption

/** 堆叠面积图配置 */
const JVMChartOptions = reactive<EChartsOption>({
  title: {
    text: 'JVM内存折线图'
  },
  dataset: {
    dimensions: ['createTime', 'runtimeMax', 'runtimeTotal', 'runtimeUsed', 'runtimeFree'],
    source: []
  },
  grid: {
    left: 20,
    right: 20,
    bottom: 0,
    top: 70,
    containLabel: true
  },
  legend: {
    top: 0
  },
  series: [
    {
      name: 'JVM最大内存', type: 'line',
      emphasis: {
        focus: 'series'
      }, smooth: false
    },
    {
      name: 'JVM占用内存', type: 'line',
      emphasis: {
        focus: 'series'
      }, smooth: false
    },
    {
      name: 'JVM可用内存', type: 'line', areaStyle: {},
      emphasis: {
        focus: 'series'
      }, smooth: false
    },
    {
      name: 'JVM空闲内存', type: 'line', areaStyle: {},
      emphasis: {
        focus: 'series'
      }, smooth: false
    }
  ],
  toolbox: {
    feature: {
      // 数据区域缩放
      dataZoom: {
        yAxisIndex: false // Y轴不缩放
      },
      brush: {
        type: ['lineX', 'clear'] // 区域缩放按钮、还原按钮
      },
      saveAsImage: {show: true, name: 'JVM内存日志图片'} // 保存为图片
    }
  },
  tooltip: {
    trigger: 'axis',
    axisPointer: {
      type: 'cross',
      label: {
        backgroundColor: '#6a7985'
      }
    },
    padding: [5, 10]
  },
  xAxis: {
    type: 'category',
    boundaryGap: false,
    axisTick: {
      show: false
    }
  },
  yAxis: {
    name: "单位（MB）",
    nameTextStyle: {
      color: "#aaa",
      nameLocation: "start",
    },
  },
}) as EChartsOption

/** 查询统计数据列表 */
const getMonitorMemDataList = async () => {
  const list = await MonitorMemApi.getMonitorMemList(queryParams)
  for (let item of list) {
    item.createTime = formatTime(item.createTime, 'yyyy-MM-dd HH:mm:ss')
  }
  // 更新 Echarts 数据
  if (physicalChartOptions.dataset && physicalChartOptions.dataset['source']) {
    physicalChartOptions.dataset['source'] = list
  }
  if (JVMChartOptions.dataset && JVMChartOptions.dataset['source']) {
    JVMChartOptions.dataset['source'] = list
  }
  echartsLoading.value = false
}

/** 切换展示方式 */
const switchShow = (type: String) => {
  showType.value = type
  if (queryParams.serverName == null && servers.value.length > 0) {
    queryParams.serverName = servers.value[0];
  }
  getDataList()
}

/** 导出按钮操作 */
const handleExport = async () => {
  try {
    // 导出的二次确认
    await message.exportConfirm()
    // 发起导出
    exportLoading.value = true
    const data = await MonitorMemApi.exportMonitorMem(queryParams)
    download.excel(data, '内存监控日志.xls')
  } catch {
  } finally {
    exportLoading.value = false
  }
}

let intervalId;
/** 初始化 **/
onMounted(() => {
  showType.value = 'data';
  getAllHost();
  getAllIp();
  getAllServer();
  getDataList()
  intervalId = setInterval(() => {
    getDataList()
  }, 60000);
})

onUnmounted(() => {
  clearInterval(intervalId);
});
</script>
